//
//  SSTableViewCell.m
//  SSTableViewSample
//
//  Created by Sa-ryong Kang on 10/5/11.
//  Copyright 2011 SeedShock, Inc. All rights reserved.
//

#import "SSTableViewCell.h"

@implementation SSTableViewCell

@synthesize identifier;
@synthesize text = _text, subtitle = _subtitle, style;
@synthesize block = _block;

+ (SSTableViewCell *)cellWithText:(NSString *)text
{
    SSTableViewCell *newCell = [[[SSTableViewCell alloc] init] autorelease];
    if (newCell) {
        newCell.text = text;
    }
    return newCell;
}

+ (SSTableViewCell *)cellWithText:(NSString *)text subtitle:(NSString *)subtitle style:(UITableViewCellStyle)style
{
    SSTableViewCell *newCell = [self cellWithText:text];
    newCell.style = style;
    newCell.subtitle = subtitle;
    return newCell;
}

- (id)init
{
    self = [super init];
    if (self) {
        style = UITableViewCellStyleDefault;
    }
    
    return self;
}

- (void)dealloc
{
    [identifier release];
    [_text release];
    [_subtitle release];
    [_block release];
    
    [super dealloc];
}

- (UITableViewCell *)generateCell
{
    NSAssert(self.identifier != nil, @"Cell identifier is nil!");
    return [[[UITableViewCell alloc] initWithStyle:style reuseIdentifier:identifier] autorelease];
}

- (void)updateCell:(UITableViewCell *)targetCell
{
    targetCell.textLabel.text = self.text;
    targetCell.detailTextLabel.text = self.subtitle;
    if (self.block)
        targetCell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    else
        targetCell.accessoryType = UITableViewCellAccessoryNone;
}

- (void)generateIdentifierBasedOn:(NSString *)baseIdentifier
{
    self.identifier = baseIdentifier;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
    
    if (self.block)
        self.block(tableView, self, indexPath);
}

@end
