//
//  SSTableViewDataSource.m
//  SSTableViewSample
//
//  Created by Sa-ryong Kang on 10/4/11.
//  Copyright 2011 SeedShock, Inc. All rights reserved.
//

#import "SSTableViewController.h"
#import "SSTableViewSection.h"
#import "SSTableViewCell.h"

@interface SSTableViewController() 

@property (retain) UITableView *superView;
@property (retain) UITableViewController *superController;

@end


@implementation SSTableViewController

@synthesize superView = _superView, superController = _superController;

#pragma mark - Initializers

- (void)setup
{
    sections = [[NSMutableArray alloc] init];
    // FIXME
    identifier = @"SSTC";
}

- (id)init
{
    self = [super init];
    [self setup];
    return self;
}

- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    [self setup];
    return self;
}

- (id)initWithTableView:(UIView *)view viewController:(UITableViewController *)controller
{
    if (![view isKindOfClass:[UITableView class]] || ![controller isKindOfClass:[UITableViewController class]]) {
        // TODO: error
        return nil;
    }
    
    self = [super init];
    if (self) {
        [self setup];
        self.superView = ((UITableView *)view);
        self.superView.dataSource = self;
        self.superView.delegate = self;
        self.superController = controller;
    }
    
    return self;
}

#pragma mark - View lifecycle

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

- (void)dealloc
{
    [sections release];
    [identifier release];
    
    [_superController release];
    [_superView release];
    
    [super dealloc];
}

#pragma mark - 

- (void)addSection:(SSTableViewSection *)section
{
    NSAssert([section isKindOfClass:[SSTableViewSection class]], @"Wrong parameter type");
    section.identifier = identifier;
    [sections addObject:section];
    section.delegate = self;
    // FIXME: CHECKME:
    [self.superView reloadData];
}

- (SSTableViewSection *)sectionAt:(NSInteger)section
{
    if (section >= [sections count]) return nil;
    SSTableViewSection *result = [sections objectAtIndex:section];
    if (![result isKindOfClass:[SSTableViewSection class]]) {
        NSAssert(NO, @"Wrong cell content");
        return nil;
    }
    return result;
}


- (SSTableViewCell *)cellAtIndexPath:(NSIndexPath *)indexPath
{
    SSTableViewSection *section = [self sectionAt:indexPath.section];
    if (section)
        return [section cellAtRow:indexPath.row];
    return nil;
}


#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return [sections count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    SSTableViewSection *sectionObject = [self sectionAt:section];
    if (sectionObject)
        return [sectionObject rowCount];
    return 0;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
    SSTableViewSection *sectionObject = [self sectionAt:section];
    if (sectionObject)
        return sectionObject.headerTitle;
    return @"";
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    SSTableViewCell *cellInfo = [self cellAtIndexPath:indexPath];

    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellInfo.identifier];
    if (cell == nil) {
        cell = [cellInfo generateCell];
    }
    
    [cellInfo updateCell:cell];
    return cell;
}

#pragma mark - UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    SSTableViewCell *cell = [self cellAtIndexPath:indexPath];
    [cell tableView:tableView didSelectRowAtIndexPath:indexPath];
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
	// create the parent view that will hold header Label
	UIView* customView = [[UIView alloc] initWithFrame:CGRectMake(10.0, 0.0, 300.0, 34.0)];
	
	// create the button object
	UILabel * headerLabel = [[UILabel alloc] initWithFrame:CGRectZero];
	headerLabel.backgroundColor = [UIColor clearColor];
	headerLabel.opaque = YES;
	headerLabel.textColor = [UIColor whiteColor];
    headerLabel.shadowColor = [UIColor darkGrayColor];
    headerLabel.shadowOffset = CGSizeMake(0, 1);
    //	headerLabel.highlightedTextColor = [UIColor whiteColor];
	headerLabel.font = [UIFont boldSystemFontOfSize:18];
	headerLabel.frame = CGRectMake(20.0, 0.0, 300.0, 34.0);
    
	// If you want to align the header text as centered
	// headerLabel.frame = CGRectMake(150.0, 0.0, 300.0, 44.0);
    
	headerLabel.text = [self tableView:tableView titleForHeaderInSection:section];
	[customView addSubview:headerLabel];
    
	return customView;
}

- (CGFloat) tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
	return 34.0;
}


#pragma mark - SSTableViewSectionUpdateDelegate protocol

- (void)updateSection:(NSUInteger)index
{
    // FIXME
    [self.superView reloadData];
}

@end
